﻿/**************************************************************
 * 
 * Copyright © Dmitry Kukushkin <codecity@gmail.com> 2007
 * All Rights Reserved
 * 
 **************************************************************/

using System;
using System.ComponentModel;
using System.Security.Permissions;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace WMSigner.WebControls
{
    [ToolboxData("<{0}:WMAcceptorInvoker runat=server />")]
    [Designer(typeof(Design.CntrDesigner))]
    [DefaultEvent("Click")]
    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class WMAcceptorInvoker : Control, INamingContainer, IPostBackEventHandler
    {
        private string _toSign;
        private string _signed;

        private HiddenField _fHash;
        private HiddenField _fWmId;
        private HiddenField _fSign;

        private event EventHandler _click;

        [Browsable(true)]
        [Bindable(false)]
        [Localizable(false)]
        public event EventHandler Click
        {
            add
            {
                if (null != _click)
                    _click += value;
                else
                    _click = value;
            }
            remove
            {
                if (null != _click)
                    _click -= value;
            }
        }

        [Browsable(true)]
        [Bindable(false)]
        [Localizable(false)]
        public string ToSign
        {
            get
            {
                return _toSign;
            }
            set
            {
                _toSign = value;
            }
        }

        [Browsable(false)]
        [Bindable(false)]
        [Localizable(false)]
        public string Signed
        {
            get
            {
                if (string.IsNullOrEmpty(_signed))
                {
                    if (null != _fHash && !string.IsNullOrEmpty(_fHash.Value))
                    {
                        _signed = Page.Session[_fHash.Value] as string;
                        Page.Session.Remove(_fHash.Value);
                    }
                }

                return _signed;
            }
        }

        [Browsable(false)]
        [Bindable(false)]
        [Localizable(false)]
        public string WmId
        {
            get
            {
                if (null == _fWmId)
                    return null;

                return _fWmId.Value;
            }
        }

        [Browsable(false)]
        [Bindable(false)]
        [Localizable(false)]
        public string Sign
        {
            get
            {
                if (null == _fSign)
                    return null;

                return _fSign.Value;
            }
        }

        protected override void OnInit(EventArgs e)
        {
            Page.ClientScript.RegisterClientScriptResource(GetType(), "WMSigner.WebControls.JS.Acceptor.js");
            Page.ClientScript.RegisterClientScriptResource(GetType(), Resource.MsgJsRes);

            StringBuilder scriptSmartSubmit = new StringBuilder();
            scriptSmartSubmit.Append("function PostBack()");
            scriptSmartSubmit.Append("{");
            scriptSmartSubmit.Append(Page.ClientScript.GetPostBackEventReference(this, string.Empty));
            scriptSmartSubmit.Append(";");
            scriptSmartSubmit.Append("}");

            if (!Page.ClientScript.IsClientScriptBlockRegistered(GetType(), "SmartSubmit"))
                Page.ClientScript.RegisterClientScriptBlock(GetType(), "SmartSubmit", scriptSmartSubmit.ToString(), true);

            string port = (80 != Page.Request.Url.Port) ? string.Format(null, ":{0}", Page.Request.Url.Port) : string.Empty;
            _toSign = string.Format(null, "<access><url>{0}{1}</url><datetime>{2}</datetime><marker>{3}</marker></access>", Page.Request.Url.Host, port, DateTime.Now.ToString(@"yyyyMMdd HH\:mm\:ss"), Guid.NewGuid());
        }

        protected override void CreateChildControls()
        {
            _fHash = new HiddenField();
            _fWmId = new HiddenField();
            _fSign = new HiddenField();

            _fHash.ID = "Hash";
            _fWmId.ID = "WmId";
            _fSign.ID = "Sign";

            Controls.Add(_fHash);
            Controls.Add(_fWmId);
            Controls.Add(_fSign);
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            string hash = _toSign.GetHashCode().ToString();

            Page.Session.Add(hash, _toSign);

            _fHash.Value = hash;
        }

        protected override void Render(HtmlTextWriter writer)
        {
            // Скрытые поля
            RenderChildren(writer);

            // ***** ActiveX WMAcceptor *****
            writer.AddAttribute(HtmlTextWriterAttribute.Id, ClientID + ClientIDSeparator + "WMAcceptorObject");
            writer.AddAttribute(HtmlTextWriterAttribute.Type, "application/x-oleobject");
            writer.AddAttribute("codebase", "https://w3s.wmtransfer.com/WMAcceptor.dll#Version=3,5,0,3");
            writer.AddAttribute(HtmlTextWriterAttribute.Height, "1");
            writer.AddAttribute(HtmlTextWriterAttribute.Width, "1");
            writer.AddAttribute("classid", "CLSID:463ED66E-431B-11D2-ADB0-0080C83DA4EB");
            writer.RenderBeginTag(HtmlTextWriterTag.Object);
            // Баг ASP.Net 2.0. Должно быть так: <param />
            writer.WriteLine(string.Format(null, "<param name=\"nState\" value=\"{0}\" />", 1));
            writer.WriteLine(string.Format(null, "<param name=\"LCID\" value=\"{0}\" />", 1049));
            writer.RenderEndTag();

            // Кнопка
            writer.AddAttribute(HtmlTextWriterAttribute.Type, "button");
            writer.AddAttribute(HtmlTextWriterAttribute.Value, Resource.BtnText);
            writer.AddAttribute(HtmlTextWriterAttribute.Onclick, string.Format(null, "Login('{0}', '{1}');", ClientID + ClientIDSeparator, ToSign));
            writer.RenderBeginTag(HtmlTextWriterTag.Input);
            writer.RenderEndTag();

            // Help
            writer.AddAttribute(HtmlTextWriterAttribute.Id, ClientID + ClientIDSeparator + "Help");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            writer.RenderEndTag();
        }

        #region IPostBackEventHandler Members

        public void RaisePostBackEvent(string eventArgument)
        {
            if (null != _click)
                _click(this, null);
        }

        #endregion
    }
}